@@ -1,10 +1,7 @@
 from __future__ import annotations
 from inspect import signature
 from typing import Any, Callable
-import os
-import warnings
 from qtpy.QtWidgets import QMenuBar, QWidget, QMainWindow, QBoxLayout
-from qtpy.QtGui import QIcon
 from qtpy.QtCore import Qt
 from magicgui.widgets import Container, MainWindow,Label, FunctionGui, Image, Table
 from magicgui.widgets._bases import Widget, ButtonWidget, ValueWidget, ContainerWidget
@@ -37,6 +34,7 @@ from ..fields import MagicField
 from ..signature import get_additional_option
 from .._app import run_app
 
+
 class ClassGuiBase(BaseGui):
     # This class is always inherited by @magicclass decorator.
     _component_class = PushButtonPlus
@@ -185,11 +183,6 @@ class ClassGuiBase(BaseGui):
                         # with a type object (not instance).
                         widget.__magicclass_parent__ = self
 
-                    elif isinstance(widget, FunctionGui):
-                        # magic-class has to know when the nested FunctionGui is called.
-                        f = nested_function_gui_callback(self, widget)
-                        widget.called.connect(f)
-                        
                     else:
                         if not widget.name:
                             widget.name = name
@@ -269,7 +262,16 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
             else:
                 object.__setattr__(self, name, value)
         
-        def _fast_insert(self: cls, key: int, widget: Widget):
+        def _fast_insert(self: cls, key: int, widget: Widget | Callable):
+            if isinstance(widget, Callable):
+                # Sometimes uses want to dynamically add new functions to GUI.
+                if isinstance(widget, FunctionGui):
+                    if widget.parent is None:
+                        f = nested_function_gui_callback(self, widget)
+                        widget.called.connect(f)
+                else:
+                    widget = self._create_widget_from_method(widget)
+                
             # _hide_labels should not contain Container because some ValueWidget like widgets
             # are Containers.
             _hide_labels = (_LabeledWidget, ButtonWidget, ClassGuiBase, FreeWidget, Label,
